@@ -10,11 +10,11 @@ Crypt::LE - Let's Encrypt API interfacing module.
 
 =head1 VERSION
 
-Version 0.03
+Version 0.04
 
 =cut
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 =head1 SYNOPSIS
 
@@ -572,10 +572,10 @@ sub request_challenge {
     return $self->_status(OK, $domains_requested ? "Requested challenges for $domains_requested domain(s)." : "There are no domains which were not yet requested for challenges.");
 }
 
-=head2 accept_challenge($callback [, $type])
+=head2 accept_challenge($callback [, $params] [, $type])
 
-Sets up a callback, which will be called for each non-verified domain to satisfy the requested challenge. Each callback will receive one parameter -
-a hash reference with the following keys:
+Sets up a callback, which will be called for each non-verified domain to satisfy the requested challenge. Each callback will receive two parameters -
+a hash reference with the challenge data and a hash reference of parameters optionally passed to accept_challenge(). The challenge data has the following keys:
 
 =over 14
 
@@ -657,12 +657,13 @@ B<Returns>: OK | ERROR | INVALID_DATA.
 
 sub accept_challenge {
     my $self = shift;
-    my ($cb, $type) = @_;
+    my ($cb, $params, $type) = @_;
     return $self->_status(ERROR, "Domains and challenges need to be set before accepting.") unless ($self->{domains} and $self->{challenges});
     my $mod_callback = ($cb and blessed $cb) ? 1 : 0;
     $type||='http';
     my $handler = "handle_challenge_$type";
     return $self->_status(INVALID_DATA, "Valid callback has not been provided.") unless ($cb and ((ref $cb eq 'CODE') or ($mod_callback and $cb->can($handler))));
+	return $self->_status(INVALID_DATA, "Passed parameters are not pointing to a hash.") if ($params and (ref $params ne 'HASH'));
     my ($domains_accepted, @domains_failed);
     $self->{active_challenges} = undef;
     foreach my $domain (sort keys %{$self->{domains}}) {
@@ -677,7 +678,7 @@ sub accept_challenge {
         my $rv;
         my $callback_data = { domain => $domain, token => $self->{challenges}->{$domain}->{$type}->{token}, fingerprint => $self->{fingerprint} };
         eval {
-            $rv = $mod_callback ? $cb->$handler($callback_data) : &$cb($callback_data); 
+            $rv = $mod_callback ? $cb->$handler($callback_data, $params) : &$cb($callback_data, $params); 
         };
         if ($@ or !$rv) {
             $self->_debug("Challenge callback for domain $domain " . ($@ ? "thrown an error: $@" : "did not return a true value"));
