@@ -27,7 +27,7 @@ sub new { bless {}, shift }
  
 sub handle_challenge_http {
     my $self = shift;
-    my $challenge = shift;
+    my ($challenge, $params) = @_;
     print "Domain: $challenge->{domain} expects '$challenge->{token}.$challenge->{fingerprint}' text in '$challenge->{token}' file\n";
     print "Press <Enter>";
     <STDIN>;
