@@ -4,18 +4,19 @@ use warnings;
 use lib '../lib';
 use Getopt::Long;
 use IO::File;
+use JSON::MaybeXS;
 use Crypt::LE ':errors';
 
 sub info { print shift, "\n" }
 sub bail { &info; exit 255 }
 
-my $VERSION = '0.03';
+my $VERSION = '0.04';
 
-my ($account_key, $csr, $csr_key, $domains, $challenge_path, $crt, $live, $generate_missing, $generate_only, $handle_with, $handle_as, $verified, $debug, $help);
+my ($account_key, $csr, $csr_key, $domains, $challenge_path, $crt, $live, $generate_missing, $generate_only, $handle_with, $handle_as, $handle_params, $verified, $debug, $help);
 
 GetOptions ("key=s" => \$account_key, "csr=s" => \$csr, "csr-key=s" => \$csr_key, "domains=s" => \$domains, "path=s" => \$challenge_path,
             "crt=s" => \$crt, "generate-missing" => \$generate_missing, "generate-only" => \$generate_only, "handle-with=s" => \$handle_with,
-	    "handle-as=s" => \$handle_as, "live" => \$live, "debug" => \$debug, "verified" => \$verified, "help" => \$help);
+	    "handle-as=s" => \$handle_as, "handle-params=s" => \$handle_params, "live" => \$live, "debug" => \$debug, "verified" => \$verified, "help" => \$help);
 
 my $usage = <<EOF;
 
@@ -33,6 +34,7 @@ Available parameters:
  path <absolute path>          - Path to local .well-known/acme-challenge/ to drop required challenge files into (optional).
  handle-with <Some::Module>    - Module name to handle challenges with (optional).
  handle-as <http|dns|tls|...>  - Type of challenge to request, by default 'http' (optional).
+ handle-params <{json}>        - JSON with parameters to be passed to the challenge-handling module (optional).
  generate-missing              - Generate missing files (key, csr and csr-key).
  generate-only                 - Generate a new key and/or CSR if they are missing and then exit.
  crt <file>                    - Name for the domain certificate file.
@@ -65,7 +67,7 @@ if ($challenge_path) {
     bail "Path to save challenge files into should be a writable directory" unless (-d $challenge_path and -w _);
 }
 
-my $handler;
+my ($handler, $params);
 $handle_as = $handle_as ? lc($handle_as) : 'http';
 
 if ($handle_with) {
@@ -74,6 +76,14 @@ if ($handle_with) {
     $handler = new $handle_with;
     my $method = 'handle_challenge_' . $handle_as;
     bail "Module to handle challenges does not seem to support the challenge type of '$handle_as'." unless $handler->can($method);
+    if ($handle_params) {
+	my $j = JSON->new->canonical()->allow_nonref();
+        eval {
+	    $params = $j->decode($handle_params);
+        };
+        bail "Could not decode parameters for the challenge-handling module: $@" if $@;
+        bail "Please make sure that your JSON document has {} around it." unless (ref $params eq 'HASH');
+    }
 }
 
 # Begin work
@@ -108,7 +118,7 @@ bail $le->error_details if $le->register;
 $le->accept_tos(); # No need to check explicitly for tos_changed() - Crypt::LE will handle that for you.
 unless ($verified) {
     bail $le->error_details if $le->request_challenge();
-    bail $le->error_details if $le->accept_challenge($handler ? $handler : \&process_challenge, $handle_as);
+    bail $le->error_details if $le->accept_challenge($handler ? $handler : \&process_challenge, $params, $handle_as);
     bail $le->error_details if $le->verify_challenge;
 }
 info "Requesting domain certificate";
@@ -119,11 +129,11 @@ if ($le->request_issuer_certificate()) {
     info "Will be saving the domain certificate alone, not the full chain (Qualys SSL test score will be capped to 'B' at best)";
     bail "Failed to save the domain certificate file" if _write($crt, $le->certificate);
 } else {
-    info "Saving the full certificate chain (With proper server configuration and HSTS you can get an 'A+' on Qualys SSL test)";
+    info "Saving the full certificate chain (With the proper server configuration and HSTS you can get an 'A+' on Qualys SSL test)";
     bail "Failed to save the domain certificate file" if _write($crt, $le->certificate . "\n" . $le->issuer);
 }
 
-info "The job is done, enjoy your certificate! For feedback and bug reports contact me at https://do-know.com";
+info "The job is done, enjoy your certificate! For feedback and bug reports contact me at [ https://do-know.com ]";
 
 sub _write {
     my ($file, $content) = @_;
